unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, StreamThreads;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    OnceButton: TButton;
    StartButton: TButton;
    StopButton: TButton;
    ResetChipButton: TButton;
    CheckTime: TTimer;
    CheckLabel: TLabel;
    SampleRateLabel: TLabel;
    SampleRateTrack: TTrackBar;
    AutoResetCheck: TCheckBox;
    TimeoutRadios: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure OnceButtonClick(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure SampleRateTrackChange(Sender: TObject);
    procedure ResetChipButtonClick(Sender: TObject);
    procedure CheckTimeTimer(Sender: TObject);
  protected
    procedure AMHz(var Msg: TMessage); message AM_HZ;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;





implementation

{$R *.DFM}

uses
  Math,
  DetectUnit, AbstractStreams;

const
  BaseHz = 27.5;
  diOnly = $FFFFFFFD; //Transcluded.

var
  DeviceIndex: LongWord;
  MyStreamThread: TStreamThread;
  bEnabledReference: Boolean = False;
  LogFN: AnsiString;
  ThisWave: TARBWave;

function TrackPosToFreq(TrackPos: Integer): Double;
begin
  Result := BaseHz * Power(2, TrackPos/12);
end;

function TimestampStr: AnsiString;
var
  ST: TSystemTime;
  Naui: TDateTime;

begin
  with ST do begin
    Naui := Now;
    DecodeDate(Naui, wYear, wMonth, wDay);
    DecodeTime(Naui, wHour, wMinute, wSecond, wMilliseconds);
    Result := Format('%.4d%.2d%.2d %.2d%.2d%.2d', [ wYear, wMonth, wDay, wHour, wMinute, wSecond ]);
  end;
end;

function BuildWave: TARBWave;
const
  SampleRate = 44100;

var
  I, L: Integer;
  Completion, F, StartF, DeltaF, T: Double;
  StartFreq, EndFreq: Double;

begin
  L := 22 * 512; //Roughly, samples in a quarter-second ripple.
  SetLength(Result, L);
  StartFreq := 2 * BaseHz;
  EndFreq := 1/2 * BaseHz;
  StartF := 2*Pi*StartFreq/SampleRate; //Scaling factors to get these frequencies at this sample rate.
  DeltaF := 2*Pi*(EndFreq - StartFreq)/SampleRate;

  T := 0;
  for I := 0 to L-1 do begin
    Completion := I/L;
    F := StartF + Completion*DeltaF;
    T := T + F;
    Result[I] := Round
    (
      $7FFF + $7FFF*Sin(T) //Convert from sine to binary offset.
      * 1/2*(1-Cos(2*Pi*Completion)) //Apply overall cosine envelope.
    );
    Result[I] := (Result[I] and $FFFE) or (I and $0001); 
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  DetectForm: TDetectForm;
  Status: LongWord;
  PID: LongWord;

begin
  Randomize;
  AutoResetCheck.Align := alTop;

  DeviceIndex := diOnly;

  Status := QueryDevicePID(DeviceIndex, PID);
  if (Status = ERROR_SUCCESS) and (OStreamTypeOfKnownCard(PID) = ostUnknown) then
    Status := ERROR_BAD_DEV_TYPE
  ;
  if Status <> ERROR_SUCCESS then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDevicePID(DeviceIndex, PID);
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  OStreamType := OStreamTypeOfKnownCard(PID);

  ThisWave := BuildWave;

  SampleRateTrackChange(SampleRateTrack);
end;

procedure TMainForm.OnceButtonClick(Sender: TObject);
var
  SampleRate: Double;

begin
  if StopButton.Enabled then StopButton.Click;

  if not bEnabledReference then begin
    //The range code doesn't matter, since the range is configured by jumpers, but setting it also enables the reference.
    EnableWrite(DeviceIndex);
    bEnabledReference := True;
  end;

  SampleRate := SampleRateTrack.Position;



  //DACOutputProcess(DeviceIndex, SampleRate, Length(ThisWave), @ThisWave[0]); //Play it once, quick and easy.
  //Play it once, quick and easy.
  OpenWriteAtFreq(DeviceIndex, SampleRate);
  WriteFrame(DeviceIndex, ThisWave);
  CloseWrite(DeviceIndex);
end;

procedure TMainForm.StartButtonClick(Sender: TObject);
var
  SampleRate: Double;
  BaseInterval, I: Integer;

begin
  if StopButton.Enabled then StopButton.Click;

  if not bEnabledReference then begin
    //The range code doesn't matter, since the range is configured by jumpers, but setting it also enables the reference.
    EnableWrite(DeviceIndex);
    bEnabledReference := True;
  end;

  SampleRate := SampleRateTrack.Position;
  BaseInterval := 1250;
  if TimeoutRadios.ItemIndex <> 0 then BaseInterval := BaseInterval * 10;

  LastReps := -1;
  Resets := 0;
  iSpeed := Round(SampleRate / 100);
  for I := Low(FreqData) to High(FreqData) do begin
    FreqData[I].Resets := 0;
    FreqData[I].Hundreds := 0;
  end;
  LogFN := TimestampStr;



  //Launch a thread to keep streaming this waveform. The thread could alternately be the one generating the waveform, so it's never the same thing twice.
  MyStreamThread := TStreamThread.Create(True);
  MyStreamThread.StreamWave := ThisWave;
  MyStreamThread.DI := DeviceIndex;
  MyStreamThread.ClockHz := SampleRate;
  MyStreamThread.Resume;

  CheckTime.Interval := Max(1, Round(22*512*4 / SampleRate * BaseInterval));
  CheckTime.Enabled := True;

  StopButton.Enabled := True;
  StartButton.Enabled := False;
  OnceButton.Enabled := False;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
var
  I: Integer;
  //RateStr: AnsiString;
  Log: TStringList;
  bSkipped: Boolean;

begin
  CheckTime.Enabled := False;

  MyStreamThread.Terminate;
  Sleep(1);
  if not MyStreamThread.bDone then repeat
    Sleep(CheckTime.Interval);
    if MyStreamThread.bDone then Break;

    ResetChip(DeviceIndex);
  until False;

  LogFN := LogFN + ' to ' + TimestampStr + '.log';

  Log := TStringList.Create;
  Log.Add('Hz'#9'Tests'#9'Resets/100');
  bSkipped := False;
  for I := Low(FreqData) to High(FreqData) do begin
    {
    if FreqData[I].Hundreds = 0 then
      RateStr := '<unknown>'
    else
      RateStr := FloatToStr(FreqData[I].Resets / FreqData[I].Hundreds)
    ;
    Log.Add(IntToStr(I * 100) + #9 + IntToStr(FreqData[I].Hundreds) + #9 + RateStr);
    }
    if FreqData[I].Hundreds <> 0 then begin
      if bSkipped then begin
        bSkipped := False;
        Log.Add('');
      end;
      Log.Add(IntToStr(I * 100) + #9 + IntToStr(FreqData[I].Hundreds) + #9 + FloatToStr(FreqData[I].Resets / FreqData[I].Hundreds));
    end
    else begin
      bSkipped := True;
    end;
  end;
  //Log.SaveToFile(LogFN);
  Log.Free;

  FreeAndNil(MyStreamThread);

  StartButton.Enabled := True;
  StopButton.Enabled := False;
  OnceButton.Enabled := True;
end;

procedure TMainForm.SampleRateTrackChange(Sender: TObject);
begin
  SampleRateLabel.Caption := 'Sample Rate: ' + IntToStr(SampleRateTrack.Position) + ' Hz';
end;

procedure TMainForm.ResetChipButtonClick(Sender: TObject);
begin
  ResetChip(DeviceIndex);
  Inc(Resets);
end;

procedure TMainForm.CheckTimeTimer(Sender: TObject);

  procedure UpdateLabel;
  begin
    CheckLabel.Caption := 'Reps: ' + IntToStr(LastReps) + #13#10'Resets: ' + IntToStr(Resets);
  end;

begin
  if not CheckTime.Enabled then Exit;

  if MyStreamThread.Reps = LastReps then begin
    if AutoResetCheck.Checked then
      ResetChipButton.Click
    ;
  end
  else
    LastReps := MyStreamThread.Reps
  ;

  UpdateLabel;
end;

procedure TMainForm.AMHz(var Msg: TMessage);
begin
  Msg.Result := 0;
  SampleRateTrack.Position := Msg.WParam;
end;

end.
